"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const objects_1 = require("../util/objects");
const Validator_1 = __importDefault(require("./Validator"));
const formData_1 = require("../util/formData");
const validator = Validator_1.default;
class BaseProxy {
    constructor(endpoint, parameters) {
        this.endpoint = endpoint;
        this.parameters = parameters;
        this.errors = Validator_1.default;
    }
    get $http() {
        return BaseProxy.$http;
    }
    all() {
        return this.submit('get', `/${this.endpoint}`);
    }
    find(id) {
        return this.submit('get', `/${this.endpoint}/${id}`);
    }
    post(payload) {
        return this.submit('post', `/${this.endpoint}`, payload);
    }
    store(payload) {
        return this.post(payload);
    }
    put(id, payload) {
        return this.submit('put', `/${this.endpoint}/${id}`, payload);
    }
    patch(id, payload) {
        return this.submit('patch', `/${this.endpoint}/${id}`, payload);
    }
    delete(id) {
        return this.submit('delete', `/${this.endpoint}/${id}`);
    }
    submit(requestType, url, form) {
        this.__validateRequestType(requestType);
        this.beforeSubmit();
        return new Promise((resolve, reject) => {
            const data = this.__hasFiles(form) ? formData_1.objectToFormData(form) : form;
            this.$http[requestType](this.__getParameterString(url), data)
                .then((response) => {
                this.onSuccess();
                const { data = {} } = response;
                resolve(data);
            })
                .catch((error) => {
                this.errors.processing = false;
                validator.processing = false;
                const { response } = error;
                if (response) {
                    const { data = {}, status } = response;
                    if (status === 422) {
                        const { errors } = data;
                        this.onFail(errors);
                        validator.fill(errors);
                    }
                    reject(error);
                }
                else {
                    reject();
                }
            });
        });
    }
    __getParameterString(url) {
        const keys = Object.keys(this.parameters);
        const parameters = keys
            .filter((key) => !!this.parameters[key])
            .map((key) => `${key}=${this.parameters[key]}`);
        return parameters.length === 0 ? url : `${url}?${parameters.join('&')}`;
    }
    __validateRequestType(requestType) {
        const requestTypes = ['get', 'delete', 'head', 'post', 'put', 'patch'];
        if (!requestTypes.includes(requestType)) {
            throw new Error(`\`${requestType}\` is not a valid request type, ` +
                `must be one of: \`${requestTypes.join('`, `')}\`.`);
        }
    }
    __hasFiles(form) {
        for (const property in form) {
            if (!form.hasOwnProperty(property)) {
                return false;
            }
            if (this.__hasFilesDeep(form[property])) {
                return true;
            }
        }
        return false;
    }
    __hasFilesDeep(object) {
        if (object === null) {
            return false;
        }
        if (typeof object === 'object') {
            for (const key in object) {
                if (object.hasOwnProperty(key)) {
                    if (objects_1.isFile(object[key])) {
                        return true;
                    }
                }
            }
        }
        if (Array.isArray(object)) {
            for (const key in object) {
                if (object.hasOwnProperty(key)) {
                    return this.__hasFilesDeep(object[key]);
                }
            }
        }
        return objects_1.isFile(object);
    }
    setParameters(parameters) {
        Object.keys(parameters).forEach((key) => {
            this.parameters[key] = parameters[key];
        });
        return this;
    }
    setParameter(parameter, value) {
        this.parameters[parameter] = value;
        return this;
    }
    removeParameters(parameters = []) {
        if (!parameters.length) {
            this.parameters = [];
        }
        else {
            parameters.forEach((parameter) => {
                delete this.parameters[parameter];
            });
        }
        return this;
    }
    removeParameter(parameter) {
        delete this.parameters[parameter];
        return this;
    }
    onFail(errors) {
        this.errors.fill(errors);
    }
    beforeSubmit() {
        if (!this.$http) {
            throw new Error('Vue Api Queries, No http library provided.');
        }
        this.errors.flush();
        this.errors.processing = true;
        this.errors.successful = false;
        validator.flush();
        validator.processing = true;
        validator.successful = false;
    }
    onSuccess() {
        this.errors.processing = false;
        this.errors.successful = true;
        validator.processing = false;
        validator.successful = true;
    }
}
exports.default = BaseProxy;
